package issueData;

import org.apache.commons.codec.digest.DigestUtils;

import java.io.Serializable;
import java.util.Objects;

public class Comment implements Serializable {

    private static final long serialVersionUID = 1L;
    private String hash;
    private String author;
    private String message;
    private long creationTime;

    public Comment(String author, String message) {
        this.author = author;
        this.message = message;
        this.creationTime = System.currentTimeMillis();
        this.hash = DigestUtils.sha256Hex(author + message + creationTime);
    }

    public String getHash() {
        return hash;
    }

    public String getAuthor() {
        return author;
    }

    public String getMessage() {
        return message;
    }

    public long getCreationTime() {
        return creationTime;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Comment comment = (Comment) o;
        return creationTime == comment.creationTime &&
                Objects.equals(hash, comment.hash) &&
                Objects.equals(author, comment.author) &&
                Objects.equals(message, comment.message);
    }

    @Override
    public int hashCode() {
        return Objects.hash(hash, author, message, creationTime);
    }
}
